@@ -111,12 +111,21 @@ static void wnck_pager_finalize    (GObject        *object);
 
 static void     wnck_pager_realize       (GtkWidget        *widget);
 static void     wnck_pager_unrealize     (GtkWidget        *widget);
+static GtkSizeRequestMode wnck_pager_get_request_mode (GtkWidget *widget);
 static void     wnck_pager_get_preferred_width (GtkWidget *widget,
                                                 int       *minimum_width,
                                                 int       *natural_width);
+static void     wnck_pager_get_preferred_width_for_height (GtkWidget *widget,
+                                                           int        height,
+                                                           int       *minimum_width,
+                                                           int       *natural_width);
 static void     wnck_pager_get_preferred_height (GtkWidget *widget,
                                                  int       *minimum_height,
                                                  int       *natural_height);
+static void     wnck_pager_get_preferred_height_for_width (GtkWidget *widget,
+                                                           int        width,
+                                                           int       *minimum_height,
+                                                           int       *natural_height);
 static void     wnck_pager_size_allocate (GtkWidget        *widget,
                                           GtkAllocation    *allocation);
 static gboolean wnck_pager_draw          (GtkWidget        *widget,
@@ -156,6 +165,8 @@ static gboolean wnck_pager_leave_notify	 (GtkWidget          *widget,
 					  GdkEventCrossing   *event);
 static gboolean wnck_pager_button_release (GtkWidget        *widget,
                                            GdkEventButton   *event);
+static gboolean wnck_pager_scroll_event  (GtkWidget        *widget,
+                                          GdkEventScroll   *event);
 static gboolean wnck_pager_focus         (GtkWidget        *widget,
                                           GtkDirectionType  direction);
 static gboolean wnck_pager_query_tooltip (GtkWidget  *widget,
@@ -250,12 +261,16 @@ wnck_pager_class_init (WnckPagerClass *klass)
 
   widget_class->realize = wnck_pager_realize;
   widget_class->unrealize = wnck_pager_unrealize;
+  widget_class->get_request_mode = wnck_pager_get_request_mode;
   widget_class->get_preferred_width = wnck_pager_get_preferred_width;
+  widget_class->get_preferred_width_for_height = wnck_pager_get_preferred_width_for_height;
   widget_class->get_preferred_height = wnck_pager_get_preferred_height;
+  widget_class->get_preferred_height_for_width = wnck_pager_get_preferred_height_for_width;
   widget_class->size_allocate = wnck_pager_size_allocate;
   widget_class->draw = wnck_pager_draw;
   widget_class->button_press_event = wnck_pager_button_press;
   widget_class->button_release_event = wnck_pager_button_release;
+  widget_class->scroll_event = wnck_pager_scroll_event;
   widget_class->motion_notify_event = wnck_pager_motion;
   widget_class->leave_notify_event = wnck_pager_leave_notify;
   widget_class->focus = wnck_pager_focus;
@@ -335,8 +350,6 @@ wnck_pager_realize (GtkWidget *widget)
   WnckPager *pager;
   GtkAllocation allocation;
   GdkWindow *window;
-  GtkStyle *style;
-  GtkStyle *new_style;
 
   pager = WNCK_PAGER (widget);
 
@@ -364,16 +377,7 @@ wnck_pager_realize (GtkWidget *widget)
   gtk_widget_set_window (widget, window);
   gdk_window_set_user_data (window, widget);
 
-  style = gtk_widget_get_style (widget);
-
-  new_style = gtk_style_attach (style, window);
-  if (new_style != style)
-    {
-      gtk_widget_set_style (widget, style);
-      style = new_style;
-    }
-
-  gtk_style_set_background (style, window, GTK_STATE_NORMAL);
+  gtk_style_context_set_background (gtk_widget_get_style_context (widget), window);
 
   /* connect to the screen of this pager. In theory, this will already have
    * been done in wnck_pager_size_request() */
@@ -404,65 +408,40 @@ wnck_pager_unrealize (GtkWidget *widget)
 }
 
 static void
-wnck_pager_size_request  (GtkWidget      *widget,
-                          GtkRequisition *requisition)
+_wnck_pager_get_padding (WnckPager *pager,
+                         GtkBorder *padding)
 {
-  WnckPager *pager;
-  int n_spaces;
-  int spaces_per_row;
-  double screen_aspect;
-  int other_dimension_size;
-  int size;
-  int n_rows;
-  int focus_width;
-  WnckWorkspace *space;
-
-  pager = WNCK_PAGER (widget);
-
-  /* if we're not realized, we don't know about our screen yet */
-  if (pager->priv->screen == NULL)
-    _wnck_pager_set_screen (pager);
-  g_assert (pager->priv->screen != NULL);
-
-  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
-
-  g_assert (pager->priv->n_rows > 0);
-  spaces_per_row = (n_spaces + pager->priv->n_rows - 1) / pager->priv->n_rows;
-  space = wnck_screen_get_workspace (pager->priv->screen, 0);
-
-  if (pager->priv->orientation == GTK_ORIENTATION_VERTICAL)
+  if (pager->priv->shadow_type != GTK_SHADOW_NONE)
     {
-      if (space) {
-        screen_aspect =
-              (double) wnck_workspace_get_height (space) /
-              (double) wnck_workspace_get_width (space);
-      } else {
-        screen_aspect =
-              (double) wnck_screen_get_height (pager->priv->screen) /
-              (double) wnck_screen_get_width (pager->priv->screen);
-      }
+      GtkWidget       *widget;
+      GtkStyleContext *context;
+      GtkStateFlags    state;
 
-      /* TODO: Handle WNCK_PAGER_DISPLAY_NAME for this case */
-
-      if (pager->priv->show_all_workspaces)
-	{
-	  size = pager->priv->workspace_size;
-	  n_rows = pager->priv->n_rows;
-	}
-      else
-	{
-	  size = pager->priv->workspace_size;
-	  n_rows = 1;
-	  spaces_per_row = 1;
-	}
-
-      other_dimension_size = screen_aspect * size;
-
-      requisition->width = size * n_rows + (n_rows - 1);
-      requisition->height = other_dimension_size * spaces_per_row  + (spaces_per_row - 1);
+      widget = GTK_WIDGET (pager);
+      state = gtk_widget_get_state_flags (widget);
+      context = gtk_widget_get_style_context (widget);
+      gtk_style_context_get_padding (context, state, padding);
     }
   else
     {
+      GtkBorder empty_padding = { 0, 0, 0, 0 };
+      *padding = empty_padding;
+    }
+}
+
+static int
+_wnck_pager_get_workspace_width_for_height (WnckPager *pager,
+                                            int        workspace_height)
+{
+  int workspace_width = 0;
+
+  if (pager->priv->display_mode == WNCK_PAGER_DISPLAY_CONTENT)
+    {
+      WnckWorkspace *space;
+      double screen_aspect;
+
+      space = wnck_screen_get_workspace (pager->priv->screen, 0);
+
       if (space) {
         screen_aspect =
               (double) wnck_workspace_get_width (space) /
@@ -473,69 +452,141 @@ wnck_pager_size_request  (GtkWidget      *widget,
               (double) wnck_screen_get_height (pager->priv->screen);
       }
 
-      if (pager->priv->show_all_workspaces)
-	{
-	  size = pager->priv->workspace_size;
-	  n_rows = pager->priv->n_rows;
-	}
-      else
-	{
-	  size = pager->priv->workspace_size;
-	  n_rows = 1;
-	  spaces_per_row = 1;
-	}
-
-      if (pager->priv->display_mode == WNCK_PAGER_DISPLAY_CONTENT)
-	{
-	  other_dimension_size = screen_aspect * size;
-	}
-      else
-	{
-	  int n_spaces, i, w;
-	  WnckScreen *screen;
-	  PangoLayout *layout;
-
-	  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
-	  layout = gtk_widget_create_pango_layout  (widget, NULL);
-	  screen = pager->priv->screen;
-	  other_dimension_size = 1;
-
-	  for (i = 0; i < n_spaces; i++)
-	    {
-	      pango_layout_set_text (layout,
-				     wnck_workspace_get_name (wnck_screen_get_workspace (screen, i)),
-				     -1);
-	      pango_layout_get_pixel_size (layout, &w, NULL);
-	      other_dimension_size = MAX (other_dimension_size, w);
-	    }
-
-	  g_object_unref (layout);
-
-	  other_dimension_size += 2;
-	}
-
-      requisition->width = other_dimension_size * spaces_per_row + (spaces_per_row - 1);
-      requisition->height = size * n_rows + (n_rows - 1);
+      workspace_width = screen_aspect * workspace_height;
     }
-
-  if (pager->priv->shadow_type != GTK_SHADOW_NONE)
+  else
     {
-      GtkStyle *style;
+      PangoLayout *layout;
+      WnckScreen *screen;
+      int n_spaces;
+      int i, w;
 
-      style = gtk_widget_get_style (widget);
+      layout = gtk_widget_create_pango_layout  (GTK_WIDGET (pager), NULL);
+      screen = pager->priv->screen;
+      n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
+      workspace_width = 1;
 
-      requisition->width += 2 * style->xthickness;
-      requisition->height += 2 * style->ythickness;
+      for (i = 0; i < n_spaces; i++)
+	{
+	  pango_layout_set_text (layout,
+				 wnck_workspace_get_name (wnck_screen_get_workspace (screen, i)),
+				 -1);
+	  pango_layout_get_pixel_size (layout, &w, NULL);
+	  workspace_width = MAX (workspace_width, w);
+	}
+
+      g_object_unref (layout);
+
+      workspace_width += 2;
     }
 
+  return workspace_width;
+}
+
+static int
+_wnck_pager_get_workspace_height_for_width (WnckPager *pager,
+                                            int        workspace_width)
+{
+  int workspace_height = 0;
+  WnckWorkspace *space;
+  double screen_aspect;
+
+  /* TODO: Handle WNCK_PAGER_DISPLAY_NAME for this case */
+
+  space = wnck_screen_get_workspace (pager->priv->screen, 0);
+
+  if (space) {
+    screen_aspect =
+	  (double) wnck_workspace_get_height (space) /
+	  (double) wnck_workspace_get_width (space);
+  } else {
+    screen_aspect =
+	  (double) wnck_screen_get_height (pager->priv->screen) /
+	  (double) wnck_screen_get_width (pager->priv->screen);
+  }
+
+  workspace_height = screen_aspect * workspace_width;
+
+  return workspace_height;
+}
+
+static void
+wnck_pager_size_request  (GtkWidget      *widget,
+                          GtkRequisition *requisition)
+{
+  WnckPager *pager;
+  int n_spaces;
+  int spaces_per_row;
+  int workspace_width, workspace_height;
+  int n_rows;
+  GtkBorder padding;
+  int focus_width;
+
+  pager = WNCK_PAGER (widget);
+
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
+  g_assert (pager->priv->n_rows > 0);
+
+  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
+
+  if (pager->priv->show_all_workspaces)
+    {
+      n_rows = pager->priv->n_rows;
+      spaces_per_row = (n_spaces + n_rows - 1) / n_rows;
+    }
+  else
+    {
+      n_rows = 1;
+      spaces_per_row = 1;
+    }
+
+  if (pager->priv->orientation == GTK_ORIENTATION_VERTICAL)
+    {
+      workspace_width = pager->priv->workspace_size;
+      workspace_height = _wnck_pager_get_workspace_height_for_width (pager,
+                                                                     workspace_width);
+
+      requisition->width = workspace_width * n_rows + (n_rows - 1);
+      requisition->height = workspace_height * spaces_per_row  + (spaces_per_row - 1);
+    }
+  else
+    {
+      workspace_height = pager->priv->workspace_size;
+      workspace_width = _wnck_pager_get_workspace_width_for_height (pager,
+                                                                    workspace_height);
+
+      requisition->width = workspace_width * spaces_per_row + (spaces_per_row - 1);
+      requisition->height = workspace_height * n_rows + (n_rows - 1);
+    }
+
+  _wnck_pager_get_padding (pager, &padding);
+  requisition->width += padding.left + padding.right;
+  requisition->height += padding.top + padding.bottom;
+
   gtk_widget_style_get (widget,
 			"focus-line-width", &focus_width,
 			NULL);
-
   requisition->width  += 2 * focus_width;
   requisition->height += 2 * focus_width;
 }
 
+static GtkSizeRequestMode
+wnck_pager_get_request_mode (GtkWidget *widget)
+{
+  WnckPager *pager;
+
+  pager = WNCK_PAGER (widget);
+
+  if (pager->priv->orientation == GTK_ORIENTATION_VERTICAL)
+    return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+  else
+    return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
+}
+
 static void
 wnck_pager_get_preferred_width (GtkWidget *widget,
                                 int       *minimum_width,
@@ -548,6 +599,63 @@ wnck_pager_get_preferred_width (GtkWidget *widget,
   *minimum_width = *natural_width = req.width;
 }
 
+static void
+wnck_pager_get_preferred_width_for_height (GtkWidget *widget,
+                                           int        height,
+                                           int       *minimum_width,
+                                           int       *natural_width)
+{
+  WnckPager *pager;
+  int n_spaces;
+  int n_rows;
+  int spaces_per_row;
+  int workspace_width, workspace_height;
+  int focus_width;
+  GtkBorder padding;
+  int width = 0;
+
+  pager = WNCK_PAGER (widget);
+
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
+  g_assert (pager->priv->n_rows > 0);
+
+  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
+
+  if (pager->priv->show_all_workspaces)
+    {
+      n_rows = pager->priv->n_rows;
+      spaces_per_row = (n_spaces + n_rows - 1) / n_rows;
+    }
+  else
+    {
+      n_rows = 1;
+      spaces_per_row = 1;
+    }
+
+  gtk_widget_style_get (widget,
+			"focus-line-width", &focus_width,
+			NULL);
+  height -= 2 * focus_width;
+  width += 2 * focus_width;
+
+  _wnck_pager_get_padding (pager, &padding);
+  height -= padding.top + padding.bottom;
+  width += padding.left + padding.right;
+
+  height -= (n_rows - 1);
+  workspace_height = height / n_rows;
+
+  workspace_width = _wnck_pager_get_workspace_width_for_height (pager,
+                                                                workspace_height);
+
+  width += workspace_width * spaces_per_row + (spaces_per_row - 1);
+  *natural_width = *minimum_width = width;
+}
+
 static void
 wnck_pager_get_preferred_height (GtkWidget *widget,
                                  int       *minimum_height,
@@ -560,6 +668,70 @@ wnck_pager_get_preferred_height (GtkWidget *widget,
   *minimum_height = *natural_height = req.height;
 }
 
+static void
+wnck_pager_get_preferred_height_for_width (GtkWidget *widget,
+                                           int        width,
+                                           int       *minimum_height,
+                                           int       *natural_height)
+{
+  WnckPager *pager;
+  int n_spaces;
+  int n_rows;
+  int spaces_per_row;
+  int workspace_width, workspace_height;
+  int focus_width;
+  GtkBorder padding;
+  int height = 0;
+
+  pager = WNCK_PAGER (widget);
+
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
+  g_assert (pager->priv->n_rows > 0);
+
+  n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
+
+  if (pager->priv->show_all_workspaces)
+    {
+      n_rows = pager->priv->n_rows;
+      spaces_per_row = (n_spaces + n_rows - 1) / n_rows;
+    }
+  else
+    {
+      n_rows = 1;
+      spaces_per_row = 1;
+    }
+
+  gtk_widget_style_get (widget,
+			"focus-line-width", &focus_width,
+			NULL);
+  width -= 2 * focus_width;
+  height += 2 * focus_width;
+
+  _wnck_pager_get_padding (pager, &padding);
+  width -= padding.left + padding.right;
+  height += padding.top + padding.bottom;
+
+  width -= (n_rows - 1);
+  workspace_width = width / n_rows;
+
+  workspace_height = _wnck_pager_get_workspace_height_for_width (pager,
+                                                                 workspace_width);
+
+  height += workspace_height * spaces_per_row + (spaces_per_row - 1);
+  *natural_height = *minimum_height = height;
+}
+
+static gboolean
+_wnck_pager_queue_resize (gpointer data)
+{
+  gtk_widget_queue_resize (GTK_WIDGET (data));
+  return FALSE;
+}
+
 static void
 wnck_pager_size_allocate (GtkWidget      *widget,
                           GtkAllocation  *allocation)
@@ -567,6 +739,7 @@ wnck_pager_size_allocate (GtkWidget      *widget,
   WnckPager *pager;
   int workspace_size;
   int focus_width;
+  GtkBorder padding;
   int width;
   int height;
 
@@ -575,19 +748,12 @@ wnck_pager_size_allocate (GtkWidget      *widget,
   gtk_widget_style_get (GTK_WIDGET (pager),
 			"focus-line-width", &focus_width,
 			NULL);
-
   width  = allocation->width  - 2 * focus_width;
-  height = allocation->height - 2* focus_width;
+  height = allocation->height - 2 * focus_width;
 
-  if (pager->priv->shadow_type != GTK_SHADOW_NONE)
-    {
-      GtkStyle *style;
-
-      style = gtk_widget_get_style (widget);
-
-      width  -= 2 * style->xthickness;
-      height -= 2 * style->ythickness;
-    }
+  _wnck_pager_get_padding (pager, &padding);
+  width  -= padding.left + padding.right;
+  height -= padding.top + padding.bottom;
 
   g_assert (pager->priv->n_rows > 0);
 
@@ -606,10 +772,12 @@ wnck_pager_size_allocate (GtkWidget      *widget,
 	workspace_size = height;
     }
 
+  workspace_size = MAX (workspace_size, 1);
+
   if (workspace_size != pager->priv->workspace_size)
     {
       pager->priv->workspace_size = workspace_size;
-      gtk_widget_queue_resize (GTK_WIDGET (widget));
+      g_idle_add (_wnck_pager_queue_resize, pager);
       return;
     }
 
@@ -628,14 +796,14 @@ get_workspace_rect (WnckPager    *pager,
   GtkWidget *widget;
   int col, row;
   GtkAllocation allocation;
-  GtkStyle *style;
+  GtkBorder padding;
   int focus_width;
 
   widget = GTK_WIDGET (pager);
 
   gtk_widget_get_allocation (widget, &allocation);
 
-  style = gtk_widget_get_style (widget);
+  _wnck_pager_get_padding (pager, &padding);
   gtk_widget_style_get (widget,
 			"focus-line-width", &focus_width,
 			NULL);
@@ -648,18 +816,10 @@ get_workspace_rect (WnckPager    *pager,
 
       if (active_space && space == wnck_workspace_get_number (active_space))
 	{
-	  rect->x = focus_width;
-	  rect->y = focus_width;
-	  rect->width = allocation.width - 2 * focus_width;
-	  rect->height = allocation.height - 2 * focus_width;
-
-	  if (pager->priv->shadow_type != GTK_SHADOW_NONE)
-	    {
-	      rect->x += style->xthickness;
-	      rect->y += style->ythickness;
-	      rect->width -= 2 * style->xthickness;
-	      rect->height -= 2 * style->ythickness;
-	    }
+	  rect->x = focus_width + padding.left;
+	  rect->y = focus_width + padding.top;
+	  rect->width = allocation.width - 2 * focus_width - padding.left - padding.right;
+	  rect->height = allocation.height - 2 * focus_width - padding.top - padding.bottom;
 	}
       else
 	{
@@ -677,8 +837,8 @@ get_workspace_rect (WnckPager    *pager,
 
   if (pager->priv->shadow_type != GTK_SHADOW_NONE)
     {
-      hsize -= 2 * style->xthickness;
-      vsize -= 2 * style->ythickness;
+      hsize -= padding.left + padding.right;
+      vsize -= padding.top + padding.bottom;
     }
 
   n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
@@ -732,8 +892,8 @@ get_workspace_rect (WnckPager    *pager,
 
   if (pager->priv->shadow_type != GTK_SHADOW_NONE)
     {
-      rect->x += style->xthickness;
-      rect->y += style->ythickness;
+      rect->x += padding.left;
+      rect->y += padding.top;
     }
 }
 
@@ -833,17 +993,17 @@ draw_window (cairo_t            *cr,
              GtkWidget          *widget,
              WnckWindow         *win,
              const GdkRectangle *winrect,
-             GtkStateType        state,
+             GtkStateFlags       state,
              gboolean            translucent)
 {
-  GtkStyle *style;
+  GtkStyleContext *context;
   GdkPixbuf *icon;
   int icon_x, icon_y, icon_w, icon_h;
   gboolean is_active;
-  GdkColor *color;
+  GdkRGBA bg, fg;
   gdouble translucency;
 
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
 
   is_active = wnck_window_is_active (win);
   translucency = translucent ? 0.4 : 1.0;
@@ -852,15 +1012,19 @@ draw_window (cairo_t            *cr,
   cairo_rectangle (cr, winrect->x, winrect->y, winrect->width, winrect->height);
   cairo_clip (cr);
 
-  if (is_active)
-    color = &style->light[state];
-  else
-    color = &style->bg[state];
-  cairo_set_source_rgba (cr,
-                         color->red / 65535.,
-                         color->green / 65535.,
-                         color->blue / 65535.,
-                         translucency);
+  gtk_style_context_get_background_color (context, state, &bg);
+  if (is_active) {
+    GtkSymbolicColor *c1, *c2;
+
+    c1 = gtk_symbolic_color_new_literal (&bg);
+    c2 = gtk_symbolic_color_new_shade (c1, 1.3);
+    gtk_symbolic_color_resolve (c2, NULL, &bg);
+    gtk_symbolic_color_unref (c2);
+    gtk_symbolic_color_unref (c1);
+  }
+
+  bg.alpha = translucency;
+  gdk_cairo_set_source_rgba (cr, &bg);
   cairo_rectangle (cr,
                    winrect->x + 1, winrect->y + 1,
                    MAX (0, winrect->width - 2), MAX (0, winrect->height - 2));
@@ -909,15 +1073,9 @@ draw_window (cairo_t            *cr,
       cairo_restore (cr);
     }
 
-  if (is_active)
-    color = &style->fg[state];
-  else
-    color = &style->fg[state];
-  cairo_set_source_rgba (cr,
-                         color->red / 65535.,
-                         color->green / 65535.,
-                         color->blue / 65535.,
-                         translucency);
+  gtk_style_context_get_color (context, state, &fg);
+  fg.alpha = translucency;
+  gdk_cairo_set_source_rgba (cr, &fg);
   cairo_set_line_width (cr, 1.0);
   cairo_rectangle (cr,
                    winrect->x + 0.5, winrect->y + 0.5,
@@ -978,8 +1136,7 @@ workspace_at_point (WnckPager *pager,
   int n_spaces;
   GtkAllocation allocation;
   int focus_width;
-  int xthickness;
-  int ythickness;
+  GtkBorder padding;
 
   widget = GTK_WIDGET (pager);
 
@@ -988,21 +1145,12 @@ workspace_at_point (WnckPager *pager,
   gtk_widget_style_get (GTK_WIDGET (pager),
 			"focus-line-width", &focus_width,
 			NULL);
+  _wnck_pager_get_padding (pager, &padding);
 
-  if (pager->priv->shadow_type != GTK_SHADOW_NONE)
-    {
-      GtkStyle *style;
-
-      style = gtk_widget_get_style (widget);
-
-      xthickness = focus_width + style->xthickness;
-      ythickness = focus_width + style->ythickness;
-    }
-  else
-    {
-      xthickness = focus_width;
-      ythickness = focus_width;
-    }
+  padding.left += focus_width;
+  padding.right += focus_width;
+  padding.top += focus_width;
+  padding.bottom += focus_width;
 
   n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
 
@@ -1019,27 +1167,27 @@ workspace_at_point (WnckPager *pager,
        * to the workspace.
        */
 
-      if (rect.x == xthickness)
+      if (rect.x == padding.left)
         {
           rect.x = 0;
-          rect.width += xthickness;
+          rect.width += padding.left;
         }
-      if (rect.y == ythickness)
+      if (rect.y == padding.top)
         {
           rect.y = 0;
-          rect.height += ythickness;
+          rect.height += padding.top;
         }
-      if (rect.y + rect.height == allocation.height - ythickness)
+      if (rect.y + rect.height == allocation.height - padding.bottom)
         {
-          rect.height += ythickness;
+          rect.height += padding.bottom;
         }
       else
         {
           rect.height += 1;
         }
-      if (rect.x + rect.width == allocation.width - xthickness)
+      if (rect.x + rect.width == allocation.width - padding.right)
         {
-          rect.width += xthickness;
+          rect.width += padding.right;
         }
       else
         {
@@ -1073,6 +1221,23 @@ workspace_at_point (WnckPager *pager,
   return -1;
 }
 
+static void
+get_dark_color_for_state (GtkStyleContext *context,
+			  GtkStateFlags    state,
+			  GdkRGBA         *dark)
+{
+  GdkRGBA           bg;
+  GtkSymbolicColor *c1, *c2;
+
+  gtk_style_context_get_background_color (context, state, &bg);
+
+  c1 = gtk_symbolic_color_new_literal (&bg);
+
+  c2 = gtk_symbolic_color_new_shade (c1, 0.7);
+  gtk_symbolic_color_resolve (c2, NULL, dark);
+  gtk_symbolic_color_unref (c2);
+  gtk_symbolic_color_unref (c1);
+}
 
 static void
 wnck_pager_draw_workspace (WnckPager    *pager,
@@ -1086,8 +1251,9 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   gboolean is_current;
   WnckWorkspace *space;
   GtkWidget *widget;
-  GtkStateType state;
-  GtkStyle *style;
+  GtkStateFlags state;
+  GtkStyleContext *context;
+  GdkRGBA color;
 
   space = wnck_screen_get_workspace (pager->priv->screen, workspace);
   if (!space)
@@ -1096,14 +1262,13 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   widget = GTK_WIDGET (pager);
   is_current = (space == wnck_screen_get_active_workspace (pager->priv->screen));
 
+  state = GTK_STATE_FLAG_NORMAL;
   if (is_current)
-    state = GTK_STATE_SELECTED;
+    state |= GTK_STATE_FLAG_SELECTED;
   else if (workspace == pager->priv->prelight)
-    state = GTK_STATE_PRELIGHT;
-  else
-    state = GTK_STATE_NORMAL;
+    state |= GTK_STATE_FLAG_PRELIGHT;
 
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
 
   /* FIXME in names mode, should probably draw things like a button.
    */
@@ -1117,7 +1282,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
     {
       if (!wnck_workspace_is_virtual (space))
         {
-          gdk_cairo_set_source_color (cr, &style->dark[state]);
+	  get_dark_color_for_state (context, state, &color);
+          gdk_cairo_set_source_rgba (cr, &color);
           cairo_rectangle (cr, rect->x, rect->y, rect->width, rect->height);
           cairo_fill (cr);
         }
@@ -1185,11 +1351,11 @@ wnck_pager_draw_workspace (WnckPager    *pager,
                         vh = rect->height + rect->y - vy;
 
                       if (active_i == i && active_j == j)
-                        gdk_cairo_set_source_color (cr,
-                                                    &style->dark[GTK_STATE_SELECTED]);
-                      else
-                        gdk_cairo_set_source_color (cr,
-                                                    &style->dark[GTK_STATE_NORMAL]);
+			get_dark_color_for_state (context, GTK_STATE_FLAG_SELECTED, &color);
+		      else
+			get_dark_color_for_state (context, GTK_STATE_FLAG_NORMAL, &color);
+
+		      gdk_cairo_set_source_rgba (cr, &color);
                       cairo_rectangle (cr, vx, vy, vw, vh);
                       cairo_fill (cr);
                     }
@@ -1201,7 +1367,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
               height_ratio = rect->height / (double) workspace_height;
 
               /* first draw non-active part of the viewport */
-              gdk_cairo_set_source_color (cr, &style->dark[GTK_STATE_NORMAL]);
+	      get_dark_color_for_state (context, GTK_STATE_FLAG_NORMAL, &color);
+              gdk_cairo_set_source_rgba (cr, &color);
               cairo_rectangle (cr, rect->x, rect->y, rect->width, rect->height);
               cairo_fill (cr);
 
@@ -1215,7 +1382,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
                   vw = width_ratio * screen_width;
                   vh = height_ratio * screen_height;
 
-                  gdk_cairo_set_source_color (cr, &style->dark[GTK_STATE_SELECTED]);
+		  get_dark_color_for_state (context, GTK_STATE_FLAG_SELECTED, &color);
+                  gdk_cairo_set_source_rgba (cr, &color);
                   cairo_rectangle (cr, vx, vy, vw, vh);
                   cairo_fill (cr);
                 }
@@ -1264,10 +1432,10 @@ wnck_pager_draw_workspace (WnckPager    *pager,
       pango_layout_get_pixel_size (layout, &w, &h);
 
       if (is_current)
-        gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_SELECTED]);
+	gtk_style_context_get_color (context, GTK_STATE_FLAG_SELECTED, &color);
       else
-        gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_NORMAL]);
-
+	gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &color);
+      gdk_cairo_set_source_rgba (cr, &color);
       cairo_move_to (cr,
                      rect->x + (rect->width - w) / 2,
                      rect->y + (rect->height - h) / 2);
@@ -1279,11 +1447,14 @@ wnck_pager_draw_workspace (WnckPager    *pager,
 
   if (workspace == pager->priv->prelight && pager->priv->prelight_dnd)
     {
-      /* stolen directly from gtk source so it matches nicely */
-      gtk_paint_shadow (style, cr,
-		        GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		        widget, "dnd",
-			rect->x, rect->y, rect->width, rect->height);
+      gtk_style_context_save (context);
+      gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+
+      cairo_save (cr);
+      gtk_render_frame (context, cr, rect->x, rect->y, rect->width, rect->height);
+      cairo_restore (cr);
+
+      gtk_style_context_restore (context);
 
       cairo_set_source_rgb (cr, 0.0, 0.0, 0.0); /* black */
       cairo_set_line_width (cr, 1.0);
@@ -1304,7 +1475,8 @@ wnck_pager_draw (GtkWidget *widget,
   WnckWorkspace *active_space;
   GdkPixbuf *bg_pixbuf;
   gboolean first;
-  GtkStyle *style;
+  GtkStyleContext *context;
+  GtkStateFlags state;
   int focus_width;
 
   pager = WNCK_PAGER (widget);
@@ -1314,35 +1486,37 @@ wnck_pager_draw (GtkWidget *widget,
   bg_pixbuf = NULL;
   first = TRUE;
 
-  style = gtk_widget_get_style (widget);
+  state = gtk_widget_get_state_flags (widget);
+  context = gtk_widget_get_style_context (widget);
   gtk_widget_style_get (widget,
 			"focus-line-width", &focus_width,
 			NULL);
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, state);
 
   if (gtk_widget_has_focus (widget))
-    gtk_paint_focus (style,
-		     cr,
-		     gtk_widget_get_state (widget),
-		     widget,
-		     "pager",
-		     0, 0,
-		     gtk_widget_get_allocated_width (widget),
-		     gtk_widget_get_allocated_height (widget));
+  {
+    cairo_save (cr);
+    gtk_render_focus (context, cr,
+		      0, 0,
+		      gtk_widget_get_allocated_width (widget),
+		      gtk_widget_get_allocated_height (widget));
+    cairo_restore (cr);
+  }
 
   if (pager->priv->shadow_type != GTK_SHADOW_NONE)
     {
-      gtk_paint_shadow (style,
-			cr,
-			gtk_widget_get_state (widget),
-			pager->priv->shadow_type,
-			widget,
-			"pager",
+      cairo_save (cr);
+      gtk_render_frame (context, cr,
 			focus_width,
 			focus_width,
                         gtk_widget_get_allocated_width (widget) - 2 * focus_width,
                         gtk_widget_get_allocated_height (widget) - 2 * focus_width);
+      cairo_restore (cr);
     }
 
+  gtk_style_context_restore (context);
+
   i = 0;
   while (i < n_spaces)
     {
@@ -1709,7 +1883,7 @@ wnck_update_drag_icon (WnckWindow     *window,
                                                rect.width, rect.height);
   cr = cairo_create (surface);
   draw_window (cr, widget, window,
-	       &rect, GTK_STATE_NORMAL, FALSE);
+	       &rect, GTK_STATE_FLAG_NORMAL, FALSE);
   cairo_destroy (cr);
   cairo_surface_set_device_offset (surface, 2, 2);
 
@@ -1907,6 +2081,91 @@ wnck_pager_button_release (GtkWidget        *widget,
   return FALSE;
 }
 
+static gboolean
+wnck_pager_scroll_event (GtkWidget      *widget,
+                         GdkEventScroll *event)
+{
+  WnckPager          *pager;
+  WnckWorkspace      *space;
+  GdkScrollDirection  absolute_direction;
+  int                 index;
+  int                 n_workspaces;
+  int                 n_columns;
+  int                 in_last_row;
+
+  pager = WNCK_PAGER (widget);
+
+  if (event->type != GDK_SCROLL)
+    return FALSE;
+
+  absolute_direction = event->direction;
+
+  space = wnck_screen_get_active_workspace (pager->priv->screen);
+  index = wnck_workspace_get_number (space);
+
+  n_workspaces = wnck_screen_get_workspace_count (pager->priv->screen);
+  n_columns = n_workspaces / pager->priv->n_rows;
+  if (n_workspaces % pager->priv->n_rows != 0)
+    n_columns++;
+  in_last_row = n_workspaces % n_columns;
+
+  if (gtk_widget_get_direction (GTK_WIDGET (pager)) == GTK_TEXT_DIR_RTL)
+    {
+      switch (event->direction)
+        {
+          case GDK_SCROLL_DOWN:
+          case GDK_SCROLL_UP:
+            break;
+          case GDK_SCROLL_RIGHT:
+            absolute_direction = GDK_SCROLL_LEFT;
+            break;
+          case GDK_SCROLL_LEFT:
+            absolute_direction = GDK_SCROLL_RIGHT;
+            break;
+        }
+    }
+
+  switch (absolute_direction)
+    {
+      case GDK_SCROLL_DOWN:
+        if (index + n_columns < n_workspaces)
+          index += n_columns;
+        else if ((index < n_workspaces - 1 &&
+                  index + in_last_row != n_workspaces - 1) ||
+                 (index == n_workspaces - 1 &&
+                  in_last_row != 0))
+          index = (index % n_columns) + 1;
+        break;
+
+      case GDK_SCROLL_RIGHT:
+        if (index < n_workspaces - 1)
+          index++;
+        break;
+
+      case GDK_SCROLL_UP:
+        if (index - n_columns >= 0)
+          index -= n_columns;
+        else if (index > 0)
+          index = ((pager->priv->n_rows - 1) * n_columns) + (index % n_columns) - 1;
+        if (index >= n_workspaces)
+          index -= n_columns;
+        break;
+
+      case GDK_SCROLL_LEFT:
+        if (index > 0)
+          index--;
+        break;
+      default:
+        g_assert_not_reached ();
+        break;
+    }
+
+  space = wnck_screen_get_workspace (pager->priv->screen, index);
+  wnck_workspace_activate (space, event->time);
+
+  return TRUE;
+}
+
 static gboolean
 wnck_pager_focus (GtkWidget        *widget,
                   GtkDirectionType  direction)
@@ -2050,6 +2309,12 @@ wnck_pager_set_layout_hint (WnckPager *pager)
  * For example, if the layout contains one row, but the orientation of the
  * layout is vertical, the #WnckPager will display a column of #WnckWorkspace.
  *
+ * Note that setting the orientation will have an effect on the geometry
+ * management: if @orientation is %GTK_ORIENTATION_HORIZONTAL,
+ * %GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT will be used as request mode; if
+ * @orientation is %GTK_ORIENTATION_VERTICAL, GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH
+ * will be used instead.
+ *
  * If @pager has not been added to a widget hierarchy, the call will fail
  * because @pager can't know the screen on which to modify the orientation.
  *
@@ -2583,9 +2848,10 @@ wnck_pager_get_background (WnckPager *pager,
 
   if (p != None)
     {
-      _wnck_error_trap_push ();
-      pix = _wnck_gdk_pixbuf_get_from_pixmap (WNCK_SCREEN_XSCREEN (pager->priv->screen), p);
-      _wnck_error_trap_pop ();
+      Screen *xscreen;
+
+      xscreen = WNCK_SCREEN_XSCREEN (pager->priv->screen);
+      pix = _wnck_gdk_pixbuf_get_from_pixmap (xscreen, p);
     }
 
   if (pix)
