@@ -3,6 +3,7 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2006-2007 Vincent Untz
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,6 +21,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if !defined (__LIBWNCK_H_INSIDE__) && !defined (WNCK_COMPILATION)
+#error "Only <libwnck/libwnck.h> can be included directly."
+#endif
+
 #ifndef WNCK_SCREEN_H
 #define WNCK_SCREEN_H
 
@@ -127,7 +132,7 @@ typedef struct _WnckWorkspaceLayout WnckWorkspaceLayout;
  * the #WnckWorkspace for each position in the layout grid, or -1 if the
  * position does not correspond to any #WnckWorkspace.
  * @grid_area: size of the grid containing all #WnckWorkspace. This can be
- * bigger than the number of #WnckWorskpace because the grid might not be
+ * bigger than the number of #WnckWorkspace because the grid might not be
  * filled.
  * @current_row: row of the specific #WnckWorkspace, starting from 0.
  * @current_col: column of the specific #WnckWorkspace, starting from 0.
@@ -137,7 +142,7 @@ typedef struct _WnckWorkspaceLayout WnckWorkspaceLayout;
  * #WnckWorkspace.
  *
  * Since: 2.12
- * Deprecated:
+ * Deprecated:2.20:
  */
 struct _WnckWorkspaceLayout
 {
@@ -150,31 +155,6 @@ struct _WnckWorkspaceLayout
 };
 #endif /* WNCK_DISABLE_DEPRECATED */
 
-/**
- * WnckMotionDirection:
- * @WNCK_MOTION_UP: search a neighbor #WnckWorkspace above another
- * #WnckWorkspace. 
- * @WNCK_MOTION_DOWN: search a neighbor #WnckWorkspace below another
- * #WnckWorkspace.
- * @WNCK_MOTION_LEFT: search a neighbor #WnckWorkspace at the left of another
- * #WnckWorkspace.
- * @WNCK_MOTION_RIGHT: search a neighbor #WnckWorkspace at the right of another
- * #WnckWorkspace.
- *
- * Type defining a direction in which to search a neighbor #WnckWorkspace.
- *
- * Since: 2.14
- */
-/* TODO: move this to workspace.h when we break API and remove
- * wnck_screen_get_workspace_neighbor() */
-typedef enum
-{
-  WNCK_MOTION_UP = -1,
-  WNCK_MOTION_DOWN = -2,
-  WNCK_MOTION_LEFT = -3,
-  WNCK_MOTION_RIGHT = -4
-} WnckMotionDirection;
-
 /**
  * WnckLayoutOrientation:
  * @WNCK_LAYOUT_ORIENTATION_HORIZONTAL: the #WnckWorkspace are laid out in
@@ -221,13 +201,6 @@ WnckScreen*    wnck_screen_get_for_root             (gulong      root_window_id)
 int            wnck_screen_get_number               (WnckScreen *screen);
 WnckWorkspace* wnck_screen_get_workspace            (WnckScreen *screen,
                                                      int         workspace);
-#ifndef WNCK_DISABLE_DEPRECATED
-int            wnck_screen_get_workspace_index      (WnckScreen    *screen,
-                                                     WnckWorkspace *space);
-WnckWorkspace* wnck_screen_get_workspace_neighbor   (WnckScreen         *screen,
-                                                     WnckWorkspace      *space,
-                                                     WnckMotionDirection direction);
-#endif /* WNCK_DISABLE_DEPRECATED */
 WnckWorkspace* wnck_screen_get_active_workspace     (WnckScreen *screen);
 GList*         wnck_screen_get_workspaces           (WnckScreen *screen);
 WnckWindow*    wnck_screen_get_active_window        (WnckScreen *screen);
