@@ -21,8 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#undef WNCK_DISABLE_DEPRECATED
-
 #include <config.h>
 
 #include <string.h>
@@ -140,7 +138,7 @@ get_action_menu (GtkWidget *widget)
 {
   while (widget) {
     if (GTK_IS_MENU_ITEM (widget))
-      widget = widget->parent;
+      widget = gtk_widget_get_parent (widget);
 
     if (WNCK_IS_ACTION_MENU (widget))
       return WNCK_ACTION_MENU (widget);
@@ -356,9 +354,11 @@ static void
 set_item_text (GtkWidget  *mi,
                const char *text)
 {
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (mi)->child),
-                      text);
-  gtk_label_set_use_underline (GTK_LABEL (GTK_BIN (mi)->child), TRUE);
+  GtkLabel *label;
+
+  label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (mi)));
+  gtk_label_set_text (label, text);
+  gtk_label_set_use_underline (label, TRUE);
 }
 
 static void
@@ -828,7 +828,7 @@ refill_submenu_workspace (WnckActionMenu *menu)
     {
       char      *name;
       GtkWidget *item;
-	
+
       name = get_workspace_name_with_accel (menu->priv->window, i);
 
       item = make_menu_item (MOVE_TO_WORKSPACE);
@@ -1223,20 +1223,3 @@ wnck_action_menu_new (WnckWindow *window)
                        "window", window,
                        NULL);
 }
-
-/**
- * wnck_create_window_action_menu:
- * @window: the #WnckWindow for which a menu will be created.
- *
- * Creates a new #WnckActionMenu. The #WnckActionMenu will be filled with menu
- * items for window operations on @window.
- *
- * Return value: a newly created #WnckActionMenu.
- *
- * Deprecated:2.22: Use wnck_action_menu_new() instead.
- */
-GtkWidget*
-wnck_create_window_action_menu (WnckWindow *window)
-{
-	return wnck_action_menu_new (window);
-}
