@@ -3,6 +3,7 @@
 
 /*
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2006-2007 Vincent Untz
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,6 +21,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if !defined (__LIBWNCK_H_INSIDE__) && !defined (WNCK_COMPILATION)
+#error "Only <libwnck/libwnck.h> can be included directly."
+#endif
+
 #ifndef WNCK_WORKSPACE_H
 #define WNCK_WORKSPACE_H
 
@@ -64,12 +69,36 @@ struct _WnckWorkspaceClass
   void (* pad4) (void);
 };
 
+/**
+ * WnckMotionDirection:
+ * @WNCK_MOTION_UP: search a neighbor #WnckWorkspace above another
+ * #WnckWorkspace. 
+ * @WNCK_MOTION_DOWN: search a neighbor #WnckWorkspace below another
+ * #WnckWorkspace.
+ * @WNCK_MOTION_LEFT: search a neighbor #WnckWorkspace at the left of another
+ * #WnckWorkspace.
+ * @WNCK_MOTION_RIGHT: search a neighbor #WnckWorkspace at the right of another
+ * #WnckWorkspace.
+ *
+ * Type defining a direction in which to search a neighbor #WnckWorkspace.
+ *
+ * Since: 2.14
+ */
+typedef enum
+{
+  WNCK_MOTION_UP = -1,
+  WNCK_MOTION_DOWN = -2,
+  WNCK_MOTION_LEFT = -3,
+  WNCK_MOTION_RIGHT = -4
+} WnckMotionDirection;
+
 GType wnck_workspace_get_type (void) G_GNUC_CONST;
 
 int         wnck_workspace_get_number     (WnckWorkspace *space);
 const char* wnck_workspace_get_name       (WnckWorkspace *space);
 void        wnck_workspace_change_name    (WnckWorkspace *space,
                                            const char    *name);
+WnckScreen* wnck_workspace_get_screen     (WnckWorkspace *space);
 void        wnck_workspace_activate       (WnckWorkspace *space,
                                            guint32        timestamp);
 int         wnck_workspace_get_width      (WnckWorkspace *space);
@@ -79,6 +108,11 @@ int         wnck_workspace_get_viewport_y (WnckWorkspace *space);
 gboolean    wnck_workspace_is_virtual     (WnckWorkspace *space);
 
 
+int wnck_workspace_get_layout_row          (WnckWorkspace       *space);
+int wnck_workspace_get_layout_column       (WnckWorkspace       *space);
+WnckWorkspace* wnck_workspace_get_neighbor (WnckWorkspace       *space,
+                                            WnckMotionDirection  direction);
+
 G_END_DECLS
 
 #endif /* WNCK_WORKSPACE_H */
